package com.recordingblogs;

import java.lang.reflect.Constructor;

/**
 * A single effect.  This class is similar to the way Orinj keeps information
 * about individual effects
 * 
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */

public class EffectStoreItem 
{
	/**
	 * The possible effect types.  The type that is listed in effect.xml for 
	 * each of the effects should match one of these types
	 */
	public final static String [] TYPE = 
		{	"delay", 			//$NON-NLS-1$
			"reverb", 			//$NON-NLS-1$
			"dynamics", 		//$NON-NLS-1$
			"equalization", 	//$NON-NLS-1$
			"filtering", 		//$NON-NLS-1$
			"other"};			//$NON-NLS-1$

	/**
	 * This class loader will be used to instantiate the effect and its dialog.
	 */
	private ClassLoader m_classLoader;
	
	/**
	 * The constructor for the actual effect.  The actual effect is a class
	 * that extends EffectUndo in oreffect.jar and implements EffectInterface
	 * in oreffect.jar.  The name of the constructor for each effect should be
	 * specified in effect.xml
	 */
	private Constructor<?> m_effectConstructor;
	
	/**
	 * The constructor for the graphical user interface for the effect.  This is 
	 * a class that extends JPanel and implements EffectPanelInterface in 
	 * oreffect.jar.  The name of the constructor for this graphical user
	 * interface should be specified in effect.xml
	 */
	private Constructor<?> m_dialogConstructor;
	
	/**
	 * The name of the effect.  This name should be specified in effect.xml
	 */
	private String m_title;
	
	/**
	 * The type of the effect.  This should be one of the types above and
	 * should be specified in effect.xml
	 */
	private String m_type;

	/**
	 * The one and only constructor
	 * @param effectConstructor The constructor for the effect
	 * @param dialogConstructor The constructor for the graphical user interface
	 *                          for the effect
	 * @param title The name of the effect
	 * @param type The type of the effect (see TYPE above)
	 */
	public EffectStoreItem(ClassLoader classLoader, Constructor<?> effectConstructor, Constructor<?> dialogConstructor, String title, String type)
	{
		m_classLoader = classLoader;
		m_effectConstructor = effectConstructor;
		m_dialogConstructor = dialogConstructor;
		m_title = title;
		
		// make sure the type exists. If not, set it to "other"
		int index = 0;
		while(index < TYPE.length && (! type.equals(TYPE[index])))
			index++;
		if (index >= TYPE.length)
			type = TYPE[TYPE.length - 1];
		m_type = type;
	}
	
	/** 
	 * Get the effect class loader
	 * @return The effect class loader
	 */
	public ClassLoader getClassLoader()
	{
		return m_classLoader;
	}

	/**
	 * Get the constructor for the effect
	 * @return The constructor for the effect
	 */
	public Constructor<?> getEffectConstructor()
	{
		return m_effectConstructor;
	}
	
	/**
	 * Get the constructor for the graphical user interface for the effect
	 * @return The constructor for the graphical user interace for the effect
	 */
	public Constructor<?> getDialogConstructor()
	{
		return m_dialogConstructor;
	}

	/**
	 * Get the name of the effect
	 * @return The name of the effect
	 */
	public String getTitle()
	{
		return m_title;
	}

	/**
	 * Get the type of the effect
	 * @return The type of the effect
	 */
	public String getType()
	{
		return m_type;
	}
}
