package com.recordingblogs;

import java.io.IOException;
import javax.sound.sampled.*;
import com.recordingblogs.oreffect.EffectChannels;
import com.recordingblogs.oreffect.EffectInterface;

/**
 * Play a single wave file and process one effect during playback.  This mixer
 * expects that the wave file is a 16-bit one
 * 
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */
public class Mixer implements Runnable
{
	/**
	 * The buffer for reading wave audio data.  Technically, in this implementation, 
	 * we do not need multiple buffers.  It may be good to pre-load the output audio
	 * device with a good amount of sound data though.  We can then continue
	 * reading sound data and feeding it to the audio device while the device is
	 * playing it from its own buffers.
	 */
	private AudioBuffer [] m_bufferPlay;
	
	/**
	 * The wave file to be played
	 */
	private WaveFile m_wave;
	
	/**
	 * The output audio device
	 */
	private SourceDataLine m_line;
	
	/**
	 * true, if the mixer should continue playing; false, otherwise.  This will
	 * become false either because the mixer reached the end of the wave file or
	 * because the user pressed Close in the EffectDialog
	 */
	private boolean m_playing;

	/**
	 * The effect to be used
	 */
	private EffectInterface m_effect;
	
	/**
	 * The current playback time
	 */
	private double m_time;
	
	/**
	 * The one and only constructor
	 */
	public Mixer()
	{
		// choose a random size for the buffers.  The only concerns here are
		// that the buffer sizes are divisible by the size of one sample (in 16-bit
		// stereo recording, one sample will have 4 bytes - 2 bytes for each channel),
		// and that the sizes of the dry buffers and wet buffers are the same
		m_bufferPlay = new AudioBuffer [4];
		for(int i = 0; i < 4; i++)
		{
			m_bufferPlay[i] = new AudioBuffer();
			m_bufferPlay[i].setSize(16384);
		}
		m_wave = null;
		m_line = null;
		m_playing = false;
		m_effect = null;
		m_time = 0;
	}

	/**
	 * Set the wave to the played
	 * @param wave  The wave to be played
	 */
	public void setWave(WaveFile wave)
	{
		m_wave = wave;
	}

	/**
	 * Set the effect to be used
	 * @param effect  The effect to be used
	 */
	public void setEffect(EffectInterface effect)
	{
		m_effect = effect;
	}

	/**
	 * Stop playback
	 */
	public void stop()
	{
		m_playing = false;
	}

	/**
	 * Start playback
	 * @return true, if there were no errors; false, otherwise
	 */
	public boolean startPlay()
	{
		// initialize the current playback time; here, we always play from the beginning
		m_time = 0;
		
		// line up the reading of the wave to the beginning of its data chunk
		m_wave.startPlay();

		// fill the buffers with silence, just in case
		for(int i = 0; i < m_bufferPlay.length; i++)
		{
			for(int j = 0; j < m_bufferPlay[i].getSize(); j++)
				m_bufferPlay[i].getBuffer()[j] = 0;
		}
		
		// read a few buffers from the wave
		for(int i = 0; i < m_bufferPlay.length; i++)
			m_wave.getData(m_bufferPlay[i]);

		// open the output audio device
		try
		{
			m_line = AudioSystem.getSourceDataLine(m_wave.getFormat());
			m_line.open();
			m_line.start();
		}
		catch (LineUnavailableException e)
		{
			System.out.println("Exception in Mixer::startPlay: " + e);		//$NON-NLS-1$
			return false;
		}

		int channels = m_wave.getFormat().getChannels();
		int blockAlign = channels * m_wave.getFormat().getSampleSizeInBits() / 8;
		int [] buffer = new int [(m_bufferPlay[0].getSize() / blockAlign) * channels];
		float value = 0;
		
		Envelope mixEnvelope = new Envelope();

		// feed the output audio device with the sound data already read
		for(int i = 0; i < m_bufferPlay.length; i++)
		{
			// convert to an integer buffer
			for(int j = 0; j < m_bufferPlay[0].getSize(); j += (blockAlign / m_wave.getFormat().getChannels()))
			{
        		buffer[j / (blockAlign / channels)] = (short) (((m_bufferPlay[i].getBuffer()[j + 1] & 0xff) << 8) + (m_bufferPlay[i].getBuffer()[j] & 0xff));
        		buffer[j / (blockAlign / channels)] = (int) (((float) buffer[j / (blockAlign / channels)] / Short.MAX_VALUE) * Integer.MAX_VALUE);
			}
			
			// apply the effect
		    m_effect.apply(buffer, null, channels, EffectChannels.CHANNEL_BOTH, m_wave.getFormat().getSampleRate(), m_time, 1F, 1F, mixEnvelope, mixEnvelope, -1D, -1D);

		    // convert back to a byte buffer 
			for(int j = 0; j < m_bufferPlay[0].getSize(); j += (blockAlign / m_wave.getFormat().getChannels()))
			{
        		value = ((float) buffer[j / (blockAlign / channels)] / Integer.MAX_VALUE) * Short.MAX_VALUE;
        		m_bufferPlay[i].getBuffer()[j] = (byte) ((int) value & 0xff);
	            m_bufferPlay[i].getBuffer()[j + 1] = (byte) ((int) value >>> 8 & 0xff);
			}
			
			m_line.write(m_bufferPlay[i].getBuffer(), 0, m_bufferPlay[i].getActualSize());
			m_time += (double) m_bufferPlay[i].getActualSize() / blockAlign;
		}

		// we are playing
		m_playing = true;
		
		return true;
	}

	/**
	 * Actual playback
	 */
	@Override
	public void run()
	{
		int channels = m_wave.getFormat().getChannels();
		int blockAlign = channels * m_wave.getFormat().getSampleSizeInBits() / 8;
		int [] buffer = new int [(m_bufferPlay[0].getSize() / blockAlign) * channels];
		float value = 0;
		Envelope mixEnvelope = new Envelope();

		// while we have not been told to stop
		while(m_playing)
		{
			// read a buffer from the wave file
			m_wave.getData(m_bufferPlay[0]);
			
			// check if this is the end of the wave file.  Orinj will also check hasData
			// you may want to fill the rest of the buffer with silence
			if (m_bufferPlay[0].getActualSize() < m_bufferPlay[0].getSize())
				m_playing = false;
			
			// convert to an integer buffer
			for(int i = 0; i < m_bufferPlay[0].getSize(); i += (blockAlign / channels))
			{
        		buffer[i / (blockAlign / channels)] = (short) (((m_bufferPlay[0].getBuffer()[i + 1] & 0xff) << 8) + (m_bufferPlay[0].getBuffer()[i] & 0xff));
        		buffer[i / (blockAlign / channels)] = (int) (((float) buffer[i / (blockAlign / channels)] / Short.MAX_VALUE) * Integer.MAX_VALUE);
			}
			
			// apply the effect
		    m_effect.apply(buffer, null, channels, EffectChannels.CHANNEL_BOTH, m_wave.getFormat().getSampleRate(), m_time, 1F, 1F, mixEnvelope, mixEnvelope, -1D, -1D);
		    
		    // convert back to a byte buffer 
			for(int i = 0; i < m_bufferPlay[0].getSize(); i += (blockAlign / channels))
			{
        		value = ((float) buffer[i / (blockAlign / channels)] / Integer.MAX_VALUE) * Short.MAX_VALUE;
        		m_bufferPlay[0].getBuffer()[i] = (byte) ((int) value & 0xff);
	            m_bufferPlay[0].getBuffer()[i + 1] = (byte) ((int) value >>> 8 & 0xff);
			}
			
			// send the result to the output audio device
			// do not use getActualSize for the buffer here, as the actual size of the 
			// last buffer may be -1
			m_line.write(m_bufferPlay[0].getBuffer(), 0, m_bufferPlay[0].getSize());
			m_time += (double) m_bufferPlay[0].getActualSize() / blockAlign;
		}

		// close the output audio device
		m_line.drain();
		m_line.stop();
		m_line.close();

		// close the wave
		try
		{
			m_wave.close();
		}
		catch (IOException e)
		{
			System.out.println("Exception in Mixer::run: " + e);		//$NON-NLS-1$
		}
	}
}
